#' Multiple comparison test using Berk and Jones (BJ) statitics.
#' @param prob - vector of input p-values.
#' @param beta - search range parameter . Beta must be between 1/n and 1.
#' @return pvalue - the p-value of the Berk-Jones test.
#' @return bjstat - the Berk-Jones statistic.
#' @return location - the order of the input p-values to obtain BJ statistic.
#' @seealso \code{\link{stat.bj}} for the definition of the statistic.
#' @references 1. Hong Zhang, Jiashun Jin and Zheyang Wu. "Distributions and Statistical Power of Optimal
#' Signal Detection Methods in Finite Samples", submitted.
#'
#' 2. Jager, Leah; Wellner, Jon A. "Goodness-of-fit tests via phi-divergences". Annals of Statistics 35 (2007).
#'
#' 3. Berk, R.H. & Jones, D.H. Z. "Goodness-of-fit test statistics that dominate the Kolmogorov statistics". Wahrscheinlichkeitstheorie verw Gebiete (1979) 47: 47.
#' @examples
#' test.bj(runif(10), 0.5)
#' #When the input are statistics#
#' stat.test = rnorm(20)
#' p.test = 1 - pnorm(stat.test)
#' test.bj(p.test, beta = 0.5)
#' @export
test.bj <- function(prob, beta){
  n = length(prob)
  result = stat.phi(prob, 1, beta)
  return(list(pvalue=1-pphi(result$value, n, 1, beta), bjstat=result$value, location=which(result$value==max(result$value))))
}
